package sun.swing;

import java.util.HashMap;
import java.awt.image.BufferedImage;
import java.awt.Image;
import java.awt.GraphicsConfiguration;
import java.awt.image.VolatileImage;
import java.awt.Graphics;
import java.awt.Component;
import java.util.Map;

public abstract class CachedPainter
{
    private static final Map<Object, ImageCache> cacheMap;
    
    private static ImageCache getCache(Object o) {
        synchronized (CachedPainter.class) {
            ImageCache imageCache = (ImageCache)CachedPainter.cacheMap.get(o);
            if (imageCache == null) {
                imageCache = new ImageCache(1);
                CachedPainter.cacheMap.put(o, imageCache);
            }
            return imageCache;
        }
    }
    
    public CachedPainter(int maxCount) {
        getCache(this.getClass()).setMaxCount(maxCount);
    }
    
    public void paint(Component component, Graphics graphics, int n, int n2, int n3, int n4, Object... array) {
        if (n3 <= 0 || n4 <= 0) {
            return;
        }
        synchronized (CachedPainter.class) {
            this.paint0(component, graphics, n, n2, n3, n4, array);
        }
        return;
    }
    
    private void paint0(Component component, Graphics graphics, int n, int n2, int n3, int n4, Object... array) {
        Class<Object> class1 = this.getClass();
        GraphicsConfiguration graphicsConfiguration = this.getGraphicsConfiguration(component);
        ImageCache cache = getCache(class1);
        Image image = cache.getImage(class1, graphicsConfiguration, n3, n4, array);
        int n5 = 0;
        do {
            boolean n6 = false;
            if ((image instanceof VolatileImage)) {
                switch (((VolatileImage)image).validate(graphicsConfiguration)) {
                    case 2: {
                        ((VolatileImage)image).flush();
                        image = null;
                        break;
                    }
                    case 1: {
                        n6 = true;
                        break;
                    }
                }
            }
            if (image == null) {
                image = this.createImage(component, n3, n4, graphicsConfiguration, array);
                cache.setImage(class1, graphicsConfiguration, n3, n4, array, image);
                n6 = true;
            }
            if (!(n6 == 0)) {
                Graphics graphics2 = image.getGraphics();
                this.paintToImage(component, image, graphics2, n3, n4, array);
                graphics2.dispose();
            }
            this.paintImage(component, graphics, n, n2, n3, n4, image, array);
        } while (image instanceof VolatileImage && ((VolatileImage)image).contentsLost() && ++n5 < 3);
    }
    
    protected abstract void paintToImage(Component p0, Image p1, Graphics p2, int p3, int p4, Object[] p5);
    
    protected void paintImage(Component component, Graphics graphics, int n, int n2, int n3, int n4, Image image, Object[] array) {
        graphics.drawImage(image, n, n2, null);
    }
    
    protected Image createImage(Component component, int n, int n2, GraphicsConfiguration graphicsConfiguration, Object[] array) {
        if (graphicsConfiguration == null) {
            return new BufferedImage(n, n2, 1);
        }
        return graphicsConfiguration.createCompatibleVolatileImage(n, n2);
    }
    
    protected void flush() {
        synchronized (CachedPainter.class) {
            getCache(this.getClass()).flush();
        }
    }
    
    private GraphicsConfiguration getGraphicsConfiguration(Component component) {
        if (component == null) {
            return null;
        }
        return component.getGraphicsConfiguration();
    }
    
    static {
        cacheMap = new HashMap();
    }
}
